% A should be positive semi-definite matrix
function [W, obj]=quadR21(A, B, r, W0)
%% quadratic loss with 21-norm regularization

%% Problem
%
%  min_{W}  Tr(W'*A*W) - 2*Tr(B'*W) + r * ||W||_21

iter = 0;
if nargin<4
    n = size(A,1);
    M = A+r*eye(n);
    W = M\B;
else
    W = W0;
end;
Wi = sqrt(sum(W.*W,2)+eps);
obj_ini = trace(W'*A*W) - 2*trace(B'*W) + r*sum(Wi);
obj_former = obj_ini;
while 1
    iter = iter + 1;
    d = 0.5./(Wi);
    D = diag(d);
    M = A+r*D;
    W = M\B;
    
    Wi = sqrt(sum(W.*W,2)+eps);
    W21 = sum(Wi);
    
    obj(iter) = trace(W'*A*W) - 2*trace(B'*W) + r*W21;
    diff = obj_former - obj(iter);
    obj_former = obj(iter);
    if diff < 10.^-3
        break;
    end
end;
obj = [obj_ini,obj];

