%% Exploit the Entire Feature Space with Sparsity 
% This sript shows how to use the SFSS algorithm for label prediction.


% References:

% Exploiting the Entire Feature Space with Sparsity for Automatic Image Annotation. 
% Zhigang Ma, Yi Yang, Feiping Nie, Jasper Uijlings, and Nicu Sebe. 
% ACM Multimedia 2011, (ACM Multimedia 11).

% Discriminating Joint Feature Analysis for Multimedia Data Understanding. 
% Zhigang Ma, Feiping Nie, Yi Yang, Jasper Uijlings, Nicu Sebe, and Alexander G. Hauptmann. 
% IEEE Transactions on Multimedia (TMM) 2012



function Y_pre = SFSS(X_tr, Y_tr, X_te, para)         
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. The training data X_tr(dim*train_num) with their labels Y_tr(train_num*class)
% 2. The testing data X_te(dim*test_num)
% 3. para : parameters
%           para.k : for computing Laplacian Matrix. 
%           para.mu : the regression parameter mu in SFSS
%           para.gamma : the L2,1 norm regularization parameter gamma in SFSS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output                    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Y_pre(test_num*class): predicted label matrix for testing data
clc;
clear;
%%%%%%%%%%%%% setting up parameters %%%%%%%%%%%%%%%
mu = para.mu;
gamma = para.gamma;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%    main script   %%%%%%%%%%%%%%%%%%%%%
[d,n] = size(X_tr);
[n,c] = size(Y_tr);
[d,nt] = size(X_te);
I = eye(n);
H = I-ones(n,1)*ones(n,1)'/n; % compute H as indicated in our paper.
%%%%%%%%%%% Compute Decision Rule Matrix U %%%%%%%%%%%%
unlabeled_id = (find(sum(Y_tr)==-c)); % c applies to the case when unlabeled data are represented as -1; 
                                      % change to 0 when unlabeled data are represented as 0;
u = 10.^10*ones(1,n);
u(unlabeled_id) = 1;
U = diag(u);
%%%%%%%%%%% Compute Lapalacian Matrix L %%%%%%%%%%%%%
L = Laplacian_GK(X_tr, para); 
%%%%%%%%%%% Following is the prediction process %%%%%%%%%
A = X_tr*H*(mu*I-mu.^2*inv(L+U+mu*H))*H*X_tr';
B = (mu*Y_tr'*U*inv(L+U+mu*H)*H*X_tr')';
[W, obj] = quadR21(A, B, gamma);
F = (L+U+mu*H)\(U*Y_tr+mu*H*X_tr'*W);
b = (F'*ones(n,1) - W'*X_tr*ones(n,1))/n;
Y_pre = X_te'*W + ones(1,nt)'*b';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%